import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
import aemo_to_tariff.energex as energex

class TestEnergex(unittest.TestCase):
    def test_feed_in_convert(self):
        interval_time = datetime(2023, 1, 15, 17, 0, tzinfo=ZoneInfo('Australia/Brisbane'))
        tariff_code = '6900'
        feed_in_price = energex.convert_feed_in_tariff(interval_time, tariff_code, 100.0)
        self.assertAlmostEqual(feed_in_price, 10.00, places=1)

    def test_convert(self):
        interval_time = datetime(2023, 7, 15, 10, 0, tzinfo=ZoneInfo('Australia/Brisbane'))
        tariff_code = '6900'
        rrp = 100.0
        expected_price = 14.065
        price = energex.convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(price, expected_price, places=2)

    def test_get_daily_fee(self):
        tariff_code = '6900'
        annual_usage = 20000
        expected_fee = 0.556
        fee = energex.get_daily_fee(tariff_code, annual_usage)
        self.assertEqual(fee, expected_fee)



import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
import aemo_to_tariff.essential as essential

class TestEssentualPower(unittest.TestCase):
    def test_some_essential_functionality(self):
        interval_time = datetime(2025, 2, 20, 9, 10, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLNT3AL'
        rrp = -100.0
        expected_price = 3.858878319999999
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.1678, expected_price, places=1)



# aemo_to_tariff/sapower.py
from datetime import time, datetime
from zoneinfo import ZoneInfo

def time_zone():
    return 'Australia/Adelaide'


def battery_tariff(customer_type: str):
    """
    Get the battery tariff for a given customer type.

    Parameters:
    - customer_type (str): The customer type ('Residential' or 'Business').

    Returns:
    - str: The battery tariff code.
    """
    if customer_type == 'Residential':
        return 'RELE2W'
    elif customer_type == 'Business':
        return 'SBTOU'
    else:
        raise ValueError("Invalid customer type. Must be 'Residential' or 'Business'.")

feed_in_tariffs = {
    'RELE2W': {
        'name': 'Residential Electrify',
        'periods': [
            ('Peak', time(17, 0), time(21, 0), 12.36),
            ('Off-peak', time(20, 0), time(6, 0), 0),
            ('Solar Sponge', time(10, 0), time(15, 0), -1)
        ],
        'peak_months': [11, 12, 1, 2, 3, 6, 7, 8]  # November–March and June–August
    }

}

tariffs = {
    'RSR': {
        'name': 'Residential Single Rate',
        'periods': [
            ('Anytime', time(0, 0), time(23, 59), 15.04)
        ]
    },
    'RTOU': {
        'name': 'Residential Time of Use',
        'periods': [
            ('Peak', time(15, 0), time(20, 0), 20.669),
            ('Peak', time(6, 0), time(10, 0), 20.669),
            ('Off-peak', time(20, 0), time(6, 0), 8.316),
            ('Solar Sponge', time(10, 0), time(15, 0), 4.191)
        ]
    },
    'RPRO': {
        'name': 'Residential Prosumer',
        'periods': [
            ('Peak', time(15, 0), time(20, 0), 20.669),
            ('Peak', time(6, 0), time(10, 0), 20.669),
            ('Off-peak', time(20, 0), time(6, 0), 8.316),
            ('Solar Sponge', time(10, 0), time(15, 0), 4.191)
        ]
    },
    'RELE': {
        'name': 'Residential Electrify',
        'periods': [
            ('Peak', time(15, 0), time(20, 0), 33.09),
            ('Peak', time(6, 0), time(10, 0), 20.669),
            ('Off-peak', time(20, 0), time(6, 0), 9.78),
            ('Solar Sponge', time(10, 0), time(15, 0), 3.01)
        ]
    },
    'RELE2W': {
        'name': 'Residential Electrify',
        'periods': [
            ('Peak', time(15, 0), time(20, 0), 33.09),
            ('Peak', time(6, 0), time(10, 0), 33.09),
            ('Off-peak', time(20, 0), time(6, 0), 9.78),
            ('Solar Sponge', time(10, 0), time(15, 0), 3.01)
        ]
    },
    'SBTOU': {
        'name': 'Small Business Time of Use',
        'periods': [
            ('Peak', time(7, 0), time(21, 0), 25.68),
            ('Off-peak', time(21, 0), time(7, 0), 9.69)
        ]
    },
    'SBTOUE': {
        'name': 'Small Business Time of Use Electrify',
        'periods': [
            ('Peak', time(7, 0), time(21, 0), 32.57),
            ('Off-peak', time(21, 0), time(7, 0), 9.60)
        ]
    }
}

daily_fees = {
    'RSR': 57.53,
    'RTOU': 57.53,
    'RPRO': 57.53,
    'RELE': 57.53,
    'SBTOU': 72.59,
    'SBTOUE': 72.59
}

demand_charges = {
    'RPRO': 83.39,  # $/kW/day
    'SBTOUD': 8.42  # $/kW/day
}


def get_periods(tariff_code: str):
    tariff = tariffs.get(tariff_code)
    if not tariff:
        raise ValueError(f"Unknown tariff code: {tariff_code}")

    return tariff['periods']

def convert_feed_in_tariff(interval_datetime: datetime, tariff_code: str, rrp: float):
    """
    Convert RRP from $/MWh to c/kWh for SA Power Networks.

    Parameters:
    - interval_datetime (datetime): The interval datetime.
    - tariff_code (str): The tariff code.
    - rrp (float): The Regional Reference Price in $/MWh.

    Returns:
    - float: The price in c/kWh.
    """
    interval_time = interval_datetime.astimezone(ZoneInfo(time_zone())).time()
    rrp_c_kwh = rrp / 10

    feed_in_tariff = feed_in_tariffs.get(tariff_code)
    if not feed_in_tariff:
        return rrp_c_kwh

    current_month = interval_datetime.month
    is_peak_month = current_month in feed_in_tariff.get('peak_months', [])

    for period_name, start, end, rate in feed_in_tariff['periods']:
        if period_name == 'Peak' and not is_peak_month:
            continue  # Skip peak period if not in peak months

        if start <= interval_time < end or (start > end and (interval_time >= start or interval_time < end)):
            total_price = rrp_c_kwh + rate
            return total_price

    return rrp_c_kwh

def convert(interval_datetime: datetime, tariff_code: str, rrp: float):
    """
    Convert RRP from $/MWh to c/kWh for SA Power Networks.

    Parameters:
    - interval_datetime (datetime): The interval datetime.
    - tariff_code (str): The tariff code.
    - rrp (float): The Regional Reference Price in $/MWh.

    Returns:
    - float: The price in c/kWh.
    """
    interval_time = interval_datetime.astimezone(ZoneInfo(time_zone())).time()
    rrp_c_kwh = rrp / 10

    tariff = tariffs.get(tariff_code)

    if not tariff:
        # Handle unknown tariff codes
        slope = 1.037869032618134
        intercept = 5.586606750833143
        return rrp_c_kwh * slope + intercept

    # Find the applicable period and rate
    for period, start, end, rate in tariff['periods']:
        if start <= interval_time < end or (start > end and (interval_time >= start or interval_time < end)):
            total_price = rrp_c_kwh + rate
            return total_price

    # If no period is found, use the first rate as default
    return rrp_c_kwh + tariff['periods'][0][3]

def get_daily_fee(tariff_code: str):
    """
    Get the daily fee for a given tariff code.

    Parameters:
    - tariff_code (str): The tariff code.

    Returns:
    - float: The daily fee in dollars.
    """
    return daily_fees.get(tariff_code, 0.0)

def calculate_demand_fee(tariff_code: str, demand_kw: float, days: int = 30):
    """
    Calculate the demand fee for a given tariff code, demand amount, and time period.

    Parameters:
    - tariff_code (str): The tariff code.
    - demand_kw (float): The maximum demand in kW.
    - days (int): The number of days for the billing period (default is 30).

    Returns:
    - float: The demand fee in dollars.
    """
    daily_charge = demand_charges.get(tariff_code, 0.0)
    return daily_charge * demand_kw * days



# setup.py

from setuptools import setup, find_packages

setup(
    name='aemo_to_tariff',
    version='0.4.5',
    description='Convert spot prices from $/MWh to c/kWh for different networks and tariffs',
    author='Ian Connor',
    author_email='ian@powston.com',
    license='MIT',
    packages=find_packages(exclude=["custom_components", "custom_components.*"]),
    install_requires=[
        'pytz',
    ],
    classifiers=[
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9',
)



from datetime import time, datetime
from zoneinfo import ZoneInfo

def time_zone():
    return 'Australia/Brisbane'

daily_fees = {
    'WRTDEMT1': 1.746,
    'ERTOUET1': 1.798,
}

demand_charges = {
    'WRTDEMT1': 22.388,
}

tariffs = {
    'WRTDEMT1': {
        'name': 'Residential Transitional Demand WRTDEMT1',
        'periods': [('Anytime', time(0, 0), time(23, 59), 26.793)],
        'rate': 26.793
    },
    'ERTOUET1': {
        'name': 'Residential ToU Energy',
        'periods': [
            ('Peak', time(17, 0), time(21, 0), 59.684),
            ('Shoulder', time(7, 0), time(17, 0), 24.912),
            ('Off-Peak', time(21, 0), time(7, 0), 7.161)
        ],
        'rate': {'Peak': 59.684, 'Shoulder': 24.912, 'Off-Peak': 7.161}
    }
}

def get_daily_fee(tariff_code: str, annual_usage: float = None):
    return daily_fees.get(tariff_code, 0.0)

def calculate_demand_fee(tariff_code: str, demand_kw: float, days: int = 30):
    if tariff_code not in demand_charges:
        return 0.0
    daily_rate = demand_charges[tariff_code]
    return daily_rate * demand_kw * days

def get_periods(tariff_code: str):
    tariff = tariffs.get(tariff_code)
    if not tariff:
        raise ValueError(f"Unknown tariff code: {tariff_code}")
    return tariff['periods']

def convert_feed_in_tariff(interval_datetime: datetime, tariff_code: str, rrp: float):
    return rrp / 10

def convert(interval_datetime: datetime, tariff_code: str, rrp: float):
    interval_time = interval_datetime.astimezone(ZoneInfo(time_zone())).time()
    rrp_c_kwh = rrp / 10
    tariff = tariffs.get(tariff_code)
    if not tariff:
        slope = 1.037869032618134
        intercept = 5.586606750833143
        return rrp_c_kwh * slope + intercept
    for period, start, end, rate in tariff['periods']:
        if start <= interval_time < end or (start > end and (interval_time >= start or interval_time < end)):
            return rrp_c_kwh + rate
    return rrp_c_kwh + (tariff['rate'] if isinstance(tariff['rate'], float) else list(tariff['rate'].values())[0])
